@@ -8,6 +8,7 @@ post linting.
 import os
 import logging
 import shutil
+import stat
 import tempfile
 from typing import (
     Any,
@@ -487,24 +488,39 @@ class LintedFile(NamedTuple):
             if suffix:
                 root, ext = os.path.splitext(fname)
                 fname = root + suffix + ext
-            self._safe_create_replace_file(fname, write_buff, self.encoding)
+            self._safe_create_replace_file(self.path, fname, write_buff, self.encoding)
         return success
 
     @staticmethod
-    def _safe_create_replace_file(fname, write_buff, encoding):
+    def _safe_create_replace_file(
+        input_path: str, output_path: str, write_buff: str, encoding: str
+    ):
         # Write to a temporary file first, so in case of encoding or other
         # issues, we don't delete or corrupt the user's existing file.
-        dirname, basename = os.path.split(fname)
+
+        # Get file mode (i.e. permissions) on existing file. We'll preserve the
+        # same permissions on the output file.
+        mode = None
+        try:
+            status = os.stat(input_path)
+        except FileNotFoundError:
+            pass
+        else:
+            if stat.S_ISREG(status.st_mode):
+                mode = stat.S_IMODE(status.st_mode)
+        dirname, basename = os.path.split(output_path)
         with tempfile.NamedTemporaryFile(
             mode="w",
             encoding=encoding,
             prefix=basename,
             dir=dirname,
-            suffix=os.path.splitext(fname)[1],
+            suffix=os.path.splitext(output_path)[1],
             delete=False,
         ) as tmp:
             tmp.file.write(write_buff)
             tmp.flush()
             os.fsync(tmp.fileno())
         # Once the temp file is safely written, replace the existing file.
-        shutil.move(tmp.name, fname)
+        if mode is not None:
+            os.chmod(tmp.name, mode)
+        shutil.move(tmp.name, output_path)
