@@ -10,6 +10,7 @@ from typing import Callable, cast, Dict, List, NamedTuple, Optional
 
 from jinja2 import Environment
 from jinja2.environment import Template
+from jinja2.exceptions import TemplateSyntaxError
 
 from sqlfluff.core.templaters.base import (
     RawFileSlice,
@@ -240,10 +241,24 @@ class JinjaAnalyzer:
             # as other code inside these regions require special handling.
             # (Generally speaking, JinjaTracer ignores the contents of these
             # blocks, treating them like opaque templated regions.)
-            filtered_trimmed_parts = [p for p in trimmed_parts if not p.isspace()]
-            if len(filtered_trimmed_parts) < 3 or filtered_trimmed_parts[2] != "=":
-                # Entering a set/macro block.
-                self.inside_set_or_macro = True
+            try:
+                # Entering a set/macro block. Build a source string consisting
+                # of just this one Jinja command and see if it parses. If so,
+                # it's a standalone command. OTOH, if it fails with "Unexpected
+                # end of template", it was the opening command for a block.
+                self.env.from_string(
+                    f"{self.env.block_start_string} {' '.join(trimmed_parts)} "
+                    f"{self.env.block_end_string}"
+                )
+            except TemplateSyntaxError as e:
+                if (
+                    isinstance(e.message, str)
+                    and "Unexpected end of template" in e.message
+                ):
+                    # It was opening a block, thus we're inside a set or macro.
+                    self.inside_set_or_macro = True
+                else:
+                    raise  # pragma: no cover
         elif block_type == "block_end" and (trimmed_parts[0] in ("endmacro", "endset")):
             # Exiting a set/macro block.
             self.inside_set_or_macro = False
@@ -294,15 +309,21 @@ class JinjaAnalyzer:
         str_parts = []
 
         # https://jinja.palletsprojects.com/en/2.11.x/api/#jinja2.Environment.lex
+        block_idx = 0
+        last_elem_type = None
         for _, elem_type, raw in self.env.lex(self.raw_str):
+            if last_elem_type == "block_end" or elem_type == "block_start":
+                block_idx += 1
+            last_elem_type = elem_type
+
             if elem_type == "data":
-                self.track_literal(raw)
+                self.track_literal(raw, block_idx)
                 continue
             str_buff += raw
             str_parts.append(raw)
 
             if elem_type.endswith("_begin"):
-                self.handle_left_whitespace_stripping(raw)
+                self.handle_left_whitespace_stripping(raw, block_idx)
 
             raw_slice_info: RawSliceInfo = self.make_raw_slice_info(None, None)
             tag_contents = []
@@ -349,16 +370,19 @@ class JinjaAnalyzer:
                             block_type,
                             self.idx_raw,
                             block_subtype,
+                            block_idx,
                         )
                     )
                     self.raw_slice_info[self.raw_sliced[-1]] = raw_slice_info
-                    block_idx = len(self.raw_sliced) - 1
+                    slice_idx = len(self.raw_sliced) - 1
                     self.idx_raw += len(str_buff) - trailing_chars
                     self.raw_sliced.append(
                         RawFileSlice(
                             str_buff[-trailing_chars:],
                             "literal",
                             self.idx_raw,
+                            None,
+                            block_idx,
                         )
                     )
                     self.raw_slice_info[
@@ -372,15 +396,16 @@ class JinjaAnalyzer:
                             block_type,
                             self.idx_raw,
                             block_subtype,
+                            block_idx,
                         )
                     )
                     self.raw_slice_info[self.raw_sliced[-1]] = raw_slice_info
-                    block_idx = len(self.raw_sliced) - 1
+                    slice_idx = len(self.raw_sliced) - 1
                     self.idx_raw += len(str_buff)
                 if block_type.startswith("block"):
                     self.track_block_end(block_type, tag_contents[0])
                     self.update_next_slice_indices(
-                        block_idx, block_type, tag_contents[0]
+                        slice_idx, block_type, tag_contents[0]
                     )
                 str_buff = ""
                 str_parts = []
@@ -407,13 +432,15 @@ class JinjaAnalyzer:
         )
         return self.make_raw_slice_info(unique_alternate_id, alternate_code)
 
-    def track_literal(self, raw: str) -> None:
+    def track_literal(self, raw: str, block_idx: int) -> None:
         """Set up tracking for a Jinja literal."""
         self.raw_sliced.append(
             RawFileSlice(
                 raw,
                 "literal",
                 self.idx_raw,
+                None,
+                block_idx,
             )
         )
         # Replace literal text with a unique ID.
@@ -482,21 +509,21 @@ class JinjaAnalyzer:
             )
 
     def update_next_slice_indices(
-        self, block_idx: int, block_type: str, tag_name: str
+        self, slice_idx: int, block_type: str, tag_name: str
     ) -> None:
         """Based on block, update conditional jump info."""
         if block_type == "block_start" and tag_name in (
             "for",
             "if",
         ):
-            self.stack.append(block_idx)
+            self.stack.append(slice_idx)
         elif block_type == "block_mid":
             # Record potential forward jump over this block.
             self.raw_slice_info[
                 self.raw_sliced[self.stack[-1]]
-            ].next_slice_indices.append(block_idx)
+            ].next_slice_indices.append(slice_idx)
             self.stack.pop()
-            self.stack.append(block_idx)
+            self.stack.append(slice_idx)
         elif block_type == "block_end" and tag_name in (
             "endfor",
             "endif",
@@ -505,15 +532,15 @@ class JinjaAnalyzer:
                 # Record potential forward jump over this block.
                 self.raw_slice_info[
                     self.raw_sliced[self.stack[-1]]
-                ].next_slice_indices.append(block_idx)
+                ].next_slice_indices.append(slice_idx)
                 if self.raw_sliced[self.stack[-1]].slice_subtype == "loop":
                     # Record potential backward jump to the loop beginning.
                     self.raw_slice_info[
-                        self.raw_sliced[block_idx]
+                        self.raw_sliced[slice_idx]
                     ].next_slice_indices.append(self.stack[-1] + 1)
                 self.stack.pop()
 
-    def handle_left_whitespace_stripping(self, token: str) -> None:
+    def handle_left_whitespace_stripping(self, token: str, block_idx: int) -> None:
         """If block open uses whitespace stripping, record it.
 
         When a "begin" tag (whether block, comment, or data) uses whitespace
@@ -553,6 +580,8 @@ class JinjaAnalyzer:
                 "Jinja lex() skipped non-whitespace: %s", skipped_str
             )
         # Treat the skipped whitespace as a literal.
-        self.raw_sliced.append(RawFileSlice(skipped_str, "literal", self.idx_raw))
+        self.raw_sliced.append(
+            RawFileSlice(skipped_str, "literal", self.idx_raw, None, block_idx)
+        )
         self.raw_slice_info[self.raw_sliced[-1]] = self.slice_info_for_literal(0)
         self.idx_raw += num_chars_skipped
