@@ -12,11 +12,13 @@ from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     AnyNumberOf,
     Anything,
+    BaseFileSegment,
     BaseSegment,
     Bracketed,
     CodeSegment,
     Dedent,
     Delimited,
+    GreedyUntil,
     Indent,
     KeywordSegment,
     Matchable,
@@ -165,6 +167,30 @@ bigquery_dialect.add(
             type="date_part",
         )
     ),
+    ProcedureNameIdentifierSegment=OneOf(
+        # In BigQuery struct() has a special syntax, so we don't treat it as a function
+        RegexParser(
+            r"[A-Z_][A-Z0-9_]*",
+            CodeSegment,
+            name="procedure_name_identifier",
+            type="procedure_name_identifier",
+            anti_template=r"STRUCT",
+        ),
+        RegexParser(
+            r"`[^`]*`",
+            CodeSegment,
+            name="procedure_name_identifier",
+            type="procedure_name_identifier",
+        ),
+    ),
+    ProcedureParameterGrammar=OneOf(
+        Sequence(
+            OneOf("IN", "OUT", "INOUT", optional=True),
+            Ref("ParameterNameSegment", optional=True),
+            OneOf(Sequence("ANY", "TYPE"), Ref("DatatypeSegment")),
+        ),
+        OneOf(Sequence("ANY", "TYPE"), Ref("DatatypeSegment")),
+    ),
 )
 
 
@@ -383,6 +409,44 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     )
 
 
+class MultiStatementSegment(BaseSegment):
+    """Overriding StatementSegment to allow for additional segment parsing."""
+
+    type = "multi_statement_segment"
+    match_grammar: Matchable = OneOf(
+        Ref("ForInStatementSegment"),
+        Ref("CreateProcedureStatementSegment"),
+    )
+
+
+class FileSegment(BaseFileSegment):
+    """A segment representing a whole file or script.
+
+    This is also the default "root" segment of the dialect,
+    and so is usually instantiated directly. It therefore
+    has no match_grammar.
+    """
+
+    # NB: We don't need a match_grammar here because we're
+    # going straight into instantiating it directly usually.
+    parse_grammar = Sequence(
+        Sequence(
+            OneOf(
+                Ref("MultiStatementSegment"),
+                Ref("StatementSegment"),
+            ),
+        ),
+        AnyNumberOf(
+            Ref("DelimiterGrammar"),
+            OneOf(
+                Ref("MultiStatementSegment"),
+                Ref("StatementSegment"),
+            ),
+        ),
+        Ref("DelimiterGrammar", optional=True),
+    )
+
+
 class StatementSegment(ansi.StatementSegment):
     """Overriding StatementSegment to allow for additional segment parsing."""
 
@@ -393,10 +457,70 @@ class StatementSegment(ansi.StatementSegment):
             Ref("SetStatementSegment"),
             Ref("ExportStatementSegment"),
             Ref("CreateExternalTableStatementSegment"),
+            Ref("AssertStatementSegment"),
+            Ref("CallStatementSegment"),
+            Ref("ReturnStatementSegment"),
+            Ref("RaiseStatementSegment"),
         ],
     )
 
 
+class AssertStatementSegment(BaseSegment):
+    """ASSERT segment.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/debugging-statements
+    """
+
+    type = "assert_statement"
+    match_grammar: Matchable = Sequence(
+        "ASSERT",
+        Ref("ExpressionSegment"),
+    )
+
+
+class ForInStatements(BaseSegment):
+    """Statements within a FOR..IN...DO...END FOR statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/procedural-language#for-in
+    """
+
+    type = "for_in_statement"
+    match_grammar = GreedyUntil(Sequence("END", "FOR"))
+    parse_grammar = AnyNumberOf(
+        Sequence(
+            Ref("StatementSegment"),
+            Ref("DelimiterGrammar"),
+        ),
+    )
+
+
+class ForInStatementSegment(BaseSegment):
+    """FOR..IN...DO...END FOR statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/procedural-language#for-in
+    """
+
+    type = "for_in_statement"
+    match_grammar = StartsWith(
+        "FOR", terminator=Sequence("END", "FOR"), include_terminator=True
+    )
+    parse_grammar = Sequence(
+        # match_grammar = Sequence(
+        "FOR",
+        Ref("SingleIdentifierGrammar"),
+        "IN",
+        Indent,
+        Ref("SelectableGrammar"),
+        Dedent,
+        "DO",
+        Indent,
+        Ref("ForInStatements"),
+        Dedent,
+        "END",
+        "FOR",
+    )
+
+
 class SelectClauseModifierSegment(ansi.SelectClauseModifierSegment):
     """Things that come after SELECT but before the columns."""
 
@@ -746,6 +870,22 @@ class TypelessStructSegment(ansi.TypelessStructSegment):
         ),
     )
 
+    # Workaround: https://github.com/sqlfluff/sqlfluff/issues/3277
+    # There is a weird issue where sometimes typeless structs are parsed as typed
+    # structs and trigger false-positives of L063 when `parse_grammar` is not set.
+    # Follow the linked issue for progress on this issue.
+    parse_grammar = Sequence(
+        "STRUCT",
+        Bracketed(
+            Delimited(
+                Sequence(
+                    Ref("BaseExpressionElementGrammar"),
+                    Ref("AliasExpressionSegment", optional=True),
+                ),
+            ),
+        ),
+    )
+
 
 class TupleSegment(BaseSegment):
     """Expression to construct a TUPLE.
@@ -1008,6 +1148,7 @@ class SetStatementSegment(BaseSegment):
                     )
                 ),
                 Ref("ArrayLiteralSegment"),
+                Ref("ExpressionSegment"),
             ),
         ),
     )
@@ -1510,3 +1651,141 @@ class ExportStatementSegment(BaseSegment):
         "AS",
         Ref("SelectableGrammar"),
     )
+
+
+class ProcedureNameSegment(BaseSegment):
+    """Prcoedure name, including any prefix bits, e.g. project or schema."""
+
+    type = "procedure_name"
+    match_grammar: Matchable = Sequence(
+        # Project name, schema identifier, etc.
+        AnyNumberOf(
+            Sequence(
+                Ref("SingleIdentifierGrammar"),
+                Ref("DotSegment"),
+            ),
+        ),
+        # Base procedure name
+        OneOf(
+            Ref("ProcedureNameIdentifierSegment"),
+            Ref("QuotedIdentifierSegment"),
+        ),
+        allow_gaps=False,
+    )
+
+
+class ProcedureParameterListSegment(BaseSegment):
+    """The parameters for a prcoedure ie. `(string, number)`."""
+
+    # Procedure parameter list (based on FunctionsParameterListGrammar)
+    type = "procedure_parameter_list"
+    match_grammar = Bracketed(
+        Delimited(
+            Ref("ProcedureParameterGrammar"),
+            delimiter=Ref("CommaSegment"),
+            bracket_pairs_set="angle_bracket_pairs",
+            optional=True,
+        )
+    )
+
+
+class ProcedureStatements(BaseSegment):
+    """Statements within a CREATE PROCEDURE statement.
+
+    https://cloud.google.com/bigquery/docs/procedures
+    """
+
+    type = "procedure_statements"
+    match_grammar = GreedyUntil("END")
+    parse_grammar = AnyNumberOf(
+        Sequence(
+            Ref("StatementSegment"),
+            Ref("DelimiterGrammar"),
+        ),
+    )
+
+
+class CreateProcedureStatementSegment(BaseSegment):
+    """A `CREATE PROCEDURE` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_procedure
+    """
+
+    type = "create_procedure_statement"
+
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        Ref("OrReplaceGrammar", optional=True),
+        "PROCEDURE",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("ProcedureNameSegment"),
+        Ref("ProcedureParameterListSegment"),
+        Sequence(
+            "OPTIONS",
+            "strict_mode",
+            StringParser(
+                "strict_mode", CodeSegment, name="strict_mode", type="procedure_option"
+            ),
+            Ref("EqualsSegment"),
+            Ref("BooleanLiteralGrammar"),
+            optional=True,
+        ),
+        "BEGIN",
+        Indent,
+        Ref("ProcedureStatements"),
+        Dedent,
+        "END",
+    )
+
+
+class CallStatementSegment(BaseSegment):
+    """A `CALL` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/procedural-language#call
+    """
+
+    type = "call_statement"
+
+    match_grammar: Matchable = Sequence(
+        "CALL",
+        Ref("ProcedureNameSegment"),
+        Bracketed(
+            Delimited(
+                Ref("ExpressionSegment"),
+                optional=True,
+            ),
+        ),
+    )
+
+
+class ReturnStatementSegment(BaseSegment):
+    """A `RETURN` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/procedural-language#return
+    """
+
+    type = "return_statement"
+
+    match_grammar: Matchable = Sequence(
+        "RETURN",
+    )
+
+
+class RaiseStatementSegment(BaseSegment):
+    """A `RAISE` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/procedural-language#raise
+    """
+
+    type = "raise_statement"
+
+    match_grammar: Matchable = Sequence(
+        "RAISE",
+        Sequence(
+            "USING",
+            "MESSAGE",
+            Ref("EqualsSegment"),
+            Ref("ExpressionSegment"),
+            optional=True,
+        ),
+    )
