@@ -106,12 +106,14 @@ ADMIN
 AFTER
 ALTER
 APPLY
+ASSERT
 AUTO_INCREMENT
 BEGIN
 BERNOULLI
 BINARY
 BINDING
 CACHE
+CALL
 CASCADE
 CHAIN
 CHARACTER
@@ -136,6 +138,7 @@ DECLARE
 DELETE
 DESCRIBE
 DETERMINISTIC
+DO
 DOMAIN
 DOUBLE
 DROP
@@ -158,8 +161,10 @@ GRANTS
 HOUR
 ILIKE
 IMPORTED
+IN
 INCREMENT
 INDEX
+INOUT
 INSERT
 INTEGRATION
 KEY
@@ -171,6 +176,7 @@ MASKING
 MATCHED
 MATERIALIZED
 MAXVALUE
+MESSAGE
 MINUS
 MINVALUE
 ML
@@ -192,6 +198,7 @@ OPERATE
 OPTION
 OPTIONS
 ORDINAL
+OUT
 OVERLAPS
 OVERWRITE
 OWNERSHIP
@@ -206,6 +213,7 @@ PROCEDURE
 PUBLIC
 QUALIFY
 QUARTER
+RAISE
 READ
 REFERENCE_USAGE
 REFERENCES
@@ -214,6 +222,7 @@ REPEATABLE
 REPLACE
 RESOURCE
 RESTRICT
+RETURN
 RETURNS
 REVOKE
 RLIKE
