@@ -7,7 +7,6 @@ ANY
 AS
 BETWEEN
 BY
-CASE
 CAST
 CHECK
 CONNECT
@@ -113,6 +112,7 @@ AT
 AUTHORIZATION
 AUTHORIZATIONS
 AUTO
+AUTO_COMPRESS
 AUTO_INCREMENT
 AUTO_INGEST
 AUTO_REFRESH
@@ -144,6 +144,7 @@ CALL
 CALLED
 CALLER
 CASCADE
+CASE
 CASE_INSENSITIVE
 CASE_SENSITIVE
 CHAIN
@@ -242,6 +243,7 @@ FUNCTION
 FUNCTIONS
 FUTURE
 GCS
+GET
 GLOBAL
 GRANTED
 GRANTS
@@ -275,9 +277,11 @@ LAST
 LAST_QUERY_ID
 LAST_TRANSACTION
 LIMIT
+LIST
 LOCAL
 LOCATION
 LOCKS
+LS
 LZO
 M
 MANAGE
@@ -336,6 +340,7 @@ OVERLAPS
 OVERWRITE
 OWNER
 OWNERSHIP
+PARALLEL
 PARAMETERS
 PARQUET
 PASSWORD
@@ -358,6 +363,7 @@ PROCEDURE
 PROCEDURES
 PUBLIC
 PURGE
+PUT
 QUERIES
 RANGE
 RAW_DEFLATE
@@ -427,6 +433,7 @@ SNAPPY
 SNAPPY_COMPRESSION
 SNOWFLAKE_FULL
 SNOWFLAKE_SSE
+SOURCE_COMPRESSION
 SQL
 STAGE
 STAGE_COPY_OPTIONS
