@@ -228,6 +228,7 @@ UNRESERVED_KEYWORDS = [
     "CALLER",
     "CAST",
     "CATCH",
+    "CODEPAGE",
     "COLUMN_ENCRYPTION_KEY",
     "COLUMNSTORE",
     "COLUMNSTORE_ARCHIVE",
@@ -240,6 +241,7 @@ UNRESERVED_KEYWORDS = [
     "CURSOR_CLOSE_ON_COMMIT",
     "CYCLE",
     "DATA_COMPRESSION",
+    "DATASOURCE",
     "DATE",
     "DATEFIRST",
     "DATEFORMAT",
@@ -255,16 +257,20 @@ UNRESERVED_KEYWORDS = [
     "ENCRYPTED",
     "ENCRYPTION",
     "ENCRYPTION_TYPE",
+    "ERRORFILE",
+    "ERRORFILE_DATA_SOURCE",
     "EXPAND",
     "EXPLAIN",  # Azure Synapse Analytics specific
     "EXPLICIT",
     "EXTERNALPUSHDOWN",
     "FAST",
+    "FIELDQUOTE",
     "FILESTREAM",
     "FILESTREAM_ON",
     "FILTER",
     "FIPS_FLAGGER",
     "FIRST",
+    "FIRSTROW",
     "FMTONLY",
     "FOLLOWING",
     "FOR",
@@ -272,6 +278,9 @@ UNRESERVED_KEYWORDS = [
     "FORCEPLAN",
     "FORCESCAN",
     "FORCESEEK",
+    "FORMAT",
+    "FORMATFILE",
+    "FORMATFILE_DATA_SOURCE",
     "HASH",
     "HEAP",  # Azure Synapse Analytics specific
     "HIDDEN",
@@ -299,6 +308,7 @@ UNRESERVED_KEYWORDS = [
     "LABEL",
     "LANGUAGE",
     "LAST",
+    "LASTROW",
     "LEVEL",
     "LOAD",  # listed as reserved but functionally unreserved
     "LOCATION",
@@ -312,6 +322,7 @@ UNRESERVED_KEYWORDS = [
     "MATCHED",
     "MAX",
     "MAXDOP",
+    "MAXERRORS",
     "MAXRECURSION",
     "MAXVALUE",
     "MIN_GRANT_PERCENT",
@@ -398,6 +409,9 @@ UNRESERVED_KEYWORDS = [
     "SHOWPLAN_ALL",
     "SHOWPLAN_TEXT",
     "SHOWPLAN_XML",
+    "SINGLE_BLOB",
+    "SINGLE_CLOB",
+    "SINGLE_NCLOB",
     "SNAPSHOT",
     "SORT_IN_TEMPDB",
     "SOURCE",
