@@ -7,7 +7,7 @@ from sqlfluff.core.parser import WhitespaceSegment
 from sqlfluff.core.parser import BaseSegment, NewlineSegment
 from sqlfluff.core.parser.segments.base import IdentitySet
 from sqlfluff.core.rules.base import BaseRule, LintFix, LintResult, RuleContext
-from sqlfluff.core.rules.doc_decorators import document_fix_compatible
+from sqlfluff.core.rules.doc_decorators import document_fix_compatible, document_groups
 from sqlfluff.core.rules.functional import Segments
 import sqlfluff.core.rules.functional.segment_predicates as sp
 
@@ -19,11 +19,13 @@ class SelectTargetsInfo(NamedTuple):
     first_new_line_idx: int
     first_select_target_idx: int
     first_whitespace_idx: int
+    comment_after_select_idx: int
     select_targets: List[BaseSegment]
     from_segment: Optional[BaseSegment]
     pre_from_whitespace: List[BaseSegment]
 
 
+@document_groups
 @document_fix_compatible
 class Rule_L036(BaseRule):
     """Select targets should be on a new line unless there is only one select target.
@@ -69,6 +71,8 @@ class Rule_L036(BaseRule):
 
     """
 
+    groups = ("all",)
+
     def _eval(self, context: RuleContext):
         if context.segment.is_type("select_clause"):
             select_targets_info = self._get_indexes(context)
@@ -88,9 +92,25 @@ class Rule_L036(BaseRule):
         select_targets = children.select(sp.is_type("select_clause_element"))
         first_select_target_idx = children.find(select_targets.get())
         selects = children.select(sp.is_keyword("select"))
-        select_idx = children.find(selects.get())
+        select_idx = children.find(selects.get()) if selects else -1
         newlines = children.select(sp.is_type("newline"))
-        first_new_line_idx = children.find(newlines.get())
+        first_new_line_idx = children.find(newlines.get()) if newlines else -1
+        comment_after_select_idx = -1
+        if newlines:
+            comment_after_select = children.select(
+                sp.is_type("comment"),
+                start_seg=selects.get(),
+                stop_seg=newlines.get(),
+                loop_while=sp.or_(
+                    sp.is_type("comment"), sp.is_type("whitespace"), sp.is_meta()
+                ),
+            )
+            if comment_after_select:
+                comment_after_select_idx = (
+                    children.find(comment_after_select.get())
+                    if comment_after_select
+                    else -1
+                )
         first_whitespace_idx = -1
         if first_new_line_idx != -1:
             # TRICKY: Ignore whitespace prior to the first newline, e.g. if
@@ -111,6 +131,7 @@ class Rule_L036(BaseRule):
             first_new_line_idx,
             first_select_target_idx,
             first_whitespace_idx,
+            comment_after_select_idx,
             select_targets,
             from_segment,
             list(pre_from_whitespace),
@@ -261,14 +282,9 @@ class Rule_L036(BaseRule):
                         """Cleans up by moving leftover select_clause segments.
 
                         Context: Some of the other fixes we make in
-                        _eval_single_select_target_element() leave the
-                        select_clause in an illegal state -- a select_clause's
-                        *rightmost children cannot be whitespace or comments*.
-                        This function addresses that by moving these segments
-                        up the parse tree to an ancestor segment chosen by
-                        _choose_anchor_segment(). After these fixes are applied,
-                        these segments may, for example, be *siblings* of
-                        select_clause.
+                        _eval_single_select_target_element() leave leftover
+                        child segments that need to be moved to become
+                        *siblings* of the select_clause.
                         """
                         start_seg = (
                             modifier[0]
@@ -299,12 +315,7 @@ class Rule_L036(BaseRule):
                         ]
                         fixes_.append(
                             LintFix.create_after(
-                                self._choose_anchor_segment(
-                                    context,
-                                    "create_after",
-                                    select_clause[0],
-                                    filter_meta=True,
-                                ),
+                                select_clause[0],
                                 ([NewlineSegment()] if add_newline else [])
                                 + list(move_after_select_clause),
                             )
@@ -382,15 +393,25 @@ class Rule_L036(BaseRule):
                             ],
                         )
 
-            fixes += [
-                # Insert the select_clause in place of the first newline in the
-                # Select statement
-                LintFix.replace(
-                    select_children[select_targets_info.first_new_line_idx],
-                    insert_buff,
-                ),
-            ]
-
+            if select_targets_info.comment_after_select_idx == -1:
+                fixes += [
+                    # Insert the select_clause in place of the first newline in the
+                    # Select statement
+                    LintFix.replace(
+                        select_children[select_targets_info.first_new_line_idx],
+                        insert_buff,
+                    ),
+                ]
+            else:
+                # The SELECT is followed by a comment on the same line. In order
+                # to autofix this, we'd need to move the select target between
+                # SELECT and the comment and potentially delete the entire line
+                # where the select target was (if it is now empty). This is
+                # *fairly tricky and complex*, in part because the newline on
+                # the select target's line is several levels higher in the
+                # parser tree. Hence, we currently don't autofix this. Could be
+                # autofixed in the future if/when we have the time.
+                fixes = []
             return LintResult(
                 anchor=select_clause.get(),
                 fixes=fixes,
